@@ -1,22 +1,22 @@
 /* Memory allocation used during tests.
 
-Copyright 2001, 2002, 2003, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -44,7 +44,7 @@ struct header {
   struct header  *next;
 };
 
-static struct header  *tests_memory_list = NULL;
+static struct header  *tests_memory_list;
 
 /* Return a pointer to a pointer to the found block (so it can be updated
    when unlinking). */
@@ -169,6 +169,7 @@ tests_free (void *ptr, size_t size)
 void
 tests_memory_start (void)
 {
+  tests_memory_list = NULL;
   mp_set_memory_functions (tests_allocate, tests_reallocate, tests_free);
 }
 
