@@ -1,22 +1,22 @@
 /* Test file for mpfr_add1sp.
 
-Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -25,8 +25,8 @@ MA 02110-1301, USA. */
 
 #include "mpfr-test.h"
 
-void check_special(void);
-void check_random(mpfr_prec_t p);
+static void check_special (void);
+static void check_random (mpfr_prec_t p);
 
 static void
 check_overflow (void)
@@ -40,7 +40,7 @@ check_overflow (void)
   set_emin (-1021);
   set_emax (1024);
 
-  mpfr_inits (x, y, z1, z2, (void *) 0);
+  mpfr_inits (x, y, z1, z2, (mpfr_ptr) 0);
 
   mpfr_set_str1 (x, "8.00468257869324898448e+307");
   mpfr_set_str1 (y, "7.44784712422708645156e+307");
@@ -51,13 +51,14 @@ check_overflow (void)
       printf ("Overflow bug in add1sp.\n");
       exit (1);
     }
-  mpfr_clears (x, y, z1, z2, (void *) 0);
+  mpfr_clears (x, y, z1, z2, (mpfr_ptr) 0);
 
   set_emin (emin);
   set_emax (emax);
 }
 
-int main(void)
+int
+main (void)
 {
   mpfr_prec_t p;
 
@@ -103,19 +104,19 @@ int main(void)
     mpfr_set_prec(b,  _p); mpfr_set_prec(c,  _p); \
   }
 
-
-void check_random(mp_prec_t p)
+static void
+check_random (mp_prec_t p)
 {
   mpfr_t a1,b,c,a2;
   int r;
   int i, inexact1, inexact2;
 
-  mpfr_inits2 (p, a1, b, c, a2, (void *) 0);
+  mpfr_inits2 (p, a1, b, c, a2, (mpfr_ptr) 0);
 
   for (i = 0 ; i < 500 ; i++)
     {
-      mpfr_random(b);
-      mpfr_random(c);
+      mpfr_urandomb (b, RANDS);
+      mpfr_urandomb (c, RANDS);
       if (MPFR_IS_PURE_FP(b) && MPFR_IS_PURE_FP(c))
         {
           if (MPFR_GET_EXP(b) < MPFR_GET_EXP(c))
@@ -133,17 +134,18 @@ void check_random(mp_prec_t p)
         }
     }
 
-  mpfr_clears (a1, a2, b, c, (void *) 0);
+  mpfr_clears (a1, a2, b, c, (mpfr_ptr) 0);
 }
 
-void check_special(void)
+static void
+check_special (void)
 {
   mpfr_t a1,a2,b,c;
   int r;
   mpfr_prec_t p;
   int i = -1, inexact1, inexact2;
 
-  mpfr_inits (a1, a2, b, c, (void *) 0);
+  mpfr_inits (a1, a2, b, c, (mpfr_ptr) 0);
 
   for (r = 0 ; r < GMP_RND_MAX ; r++)
     {
@@ -179,5 +181,5 @@ void check_special(void)
       exit (1);
     }
 
-  mpfr_clears (a1, a2, b, c, (void *) 0);
+  mpfr_clears (a1, a2, b, c, (mpfr_ptr) 0);
 }
