@@ -1,22 +1,22 @@
 /* Test file for mpfr_cmp_ld.
 
-Copyright 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2004, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 #include <stdio.h>
@@ -34,18 +34,18 @@ main (void)
 
   mpfr_init2(x, MPFR_LDBL_MANT_DIG);
 
-  mpfr_set_ld (x, 2.34763465, GMP_RNDN);
-  if (mpfr_cmp_ld(x, 2.34763465)!=0) {
+  mpfr_set_ld (x, 2.34763465L, GMP_RNDN);
+  if (mpfr_cmp_ld(x, 2.34763465L)!=0) {
     printf("Error in mpfr_cmp_ld 2.34763465 and ");
     mpfr_out_str(stdout, 10, 0, x, GMP_RNDN); putchar('\n');
     exit(1);
   }
-  if (mpfr_cmp_ld(x, 2.345)<=0) {
+  if (mpfr_cmp_ld(x, 2.345L)<=0) {
     printf("Error in mpfr_cmp_ld 2.345 and ");
     mpfr_out_str(stdout, 10, 0, x, GMP_RNDN); putchar('\n');
     exit(1);
   }
-  if (mpfr_cmp_ld(x, 2.4)>=0) {
+  if (mpfr_cmp_ld(x, 2.4L)>=0) {
     printf("Error in mpfr_cmp_ld 2.4 and ");
     mpfr_out_str(stdout, 10, 0, x, GMP_RNDN); putchar('\n');
     exit(1);
@@ -73,6 +73,11 @@ main (void)
   if (c != 0 || !mpfr_erangeflag_p ())
     {
       printf ("ERROR for NAN (1)\n");
+#ifdef MPFR_NANISNAN
+      printf ("The reason is that NAN == NAN. Please look at the configure "
+              "output\nand Section \"In case of problem\" of the INSTALL "
+              "file.\n");
+#endif
       exit (1);
     }
   mpfr_set_nan (x);
@@ -81,10 +86,14 @@ main (void)
   if (c != 0 || !mpfr_erangeflag_p ())
     {
       printf ("ERROR for NAN (2)\n");
+#ifdef MPFR_NANISNAN
+      printf ("The reason is that NAN == NAN. Please look at the configure "
+              "output\nand Section \"In case of problem\" of the INSTALL "
+              "file.\n");
+#endif
       exit (1);
     }
 
-
   mpfr_clear(x);
 
   tests_end_mpfr ();
