@@ -1,22 +1,22 @@
 /* Test file for mpfr_pow_z -- power function x^z with z a MPZ
 
-Copyright 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -47,7 +47,7 @@ check_special (void)
     ERROR ("23^0");
   mpfr_set_nan (x);
   res = mpfr_pow_z (y, x, z, GMP_RNDN);
-  if (res != 0 || mpfr_nan_p (y) == 0)
+  if (res != 0 || mpfr_nan_p (y) || mpfr_cmp_si (y, 1) != 0)
     ERROR ("NAN^0");
   mpfr_set_inf (x, 1);
   res = mpfr_pow_z (y, x, z, GMP_RNDN);
@@ -147,7 +147,7 @@ check_integer (mp_prec_t begin, mp_prec_t end, unsigned long max)
   int res1, res2;
   mp_rnd_t rnd;
 
-  mpfr_inits2 (begin, x, y1, y2, (void *) 0);
+  mpfr_inits2 (begin, x, y1, y2, (mpfr_ptr) 0);
   mpz_init (z);
   for (p = begin ; p < end ; p+=4)
     {
@@ -156,10 +156,12 @@ check_integer (mp_prec_t begin, mp_prec_t end, unsigned long max)
       mpfr_set_prec (y2, p);
       for (i = 0 ; i < max ; i++)
         {
-          mpz_random (z, (i&1) == 0 ? -1 : 1);
-          mpfr_random (x);
+          mpz_urandomb (z, RANDS, GMP_NUMB_BITS);
+          if ((i & 1) != 0)
+            mpz_neg (z, z);
+          mpfr_urandomb (x, RANDS);
           mpfr_mul_2ui (x, x, 1, GMP_RNDN); /* 0 <= x < 2 */
-          rnd = (mp_rnd_t) RND_RAND ();
+          rnd = RND_RAND ();
           if (mpz_fits_slong_p (z))
             {
               n = mpz_get_si (z);
@@ -169,7 +171,7 @@ check_integer (mp_prec_t begin, mp_prec_t end, unsigned long max)
               res2 = mpfr_pow_z  (y2, x, z, rnd);
               if (mpfr_cmp (y1, y2) != 0)
                 {
-                  printf ("Error for p = %lu, z = %ld, rnd = %s and x = ",
+                  printf ("Error for p = %lu, z = %lu, rnd = %s and x = ",
                           p, n, mpfr_print_rnd_mode (rnd));
                   mpfr_dump (x);
                   printf ("Ypowsi = "); mpfr_dump (y1);
@@ -178,7 +180,7 @@ check_integer (mp_prec_t begin, mp_prec_t end, unsigned long max)
                 }
               if (res1 != res2)
                 {
-                  printf ("Wrong inexact flags for p = %lu, z = %ld, rnd = %s"
+                  printf ("Wrong inexact flags for p = %lu, z = %lu, rnd = %s"
                           " and x = ", p, n, mpfr_print_rnd_mode (rnd));
                   mpfr_dump (x);
                   printf ("Ypowsi(inex = %2d) = ", res1); mpfr_dump (y1);
@@ -188,7 +190,7 @@ check_integer (mp_prec_t begin, mp_prec_t end, unsigned long max)
             }
         } /* for i */
     } /* for p */
-  mpfr_clears (x, y1, y2, (void *) 0);
+  mpfr_clears (x, y1, y2, (mpfr_ptr) 0);
   mpz_clear (z);
 }
 
@@ -229,7 +231,7 @@ bug20071104 (void)
   int inex;
 
   mpz_init_set_si (z, -2);
-  mpfr_inits2 (20, x, y, (void *) 0);
+  mpfr_inits2 (20, x, y, (mpfr_ptr) 0);
   mpfr_set_ui (x, 0, GMP_RNDN);
   mpfr_nextbelow (x);  /* x = -2^(emin-1) */
   mpfr_clear_flags ();
@@ -250,7 +252,7 @@ bug20071104 (void)
       printf ("Error in bug20071104: bad flags (%u)\n", __gmpfr_flags);
       exit (1);
     }
-  mpfr_clears (x, y, (void *) 0);
+  mpfr_clears (x, y, (mpfr_ptr) 0);
   mpz_clear (z);
 }
 
@@ -293,16 +295,71 @@ check_overflow (void)
   mpz_clear (z);
 }
 
+/* bug reported by Carl Witty (32-bit architecture) */
+static void
+bug20080223 (void)
+{
+  mpfr_t a, exp, answer;
+
+  mpfr_init2 (a, 53);
+  mpfr_init2 (exp, 53);
+  mpfr_init2 (answer, 53);
+
+  mpfr_set_si (exp, -1073741824, GMP_RNDN);
+
+  mpfr_set_str (a, "1.999999999", 10, GMP_RNDN);
+  /* a = 562949953139837/2^48 */
+  mpfr_pow (answer, a, exp, GMP_RNDN);
+  mpfr_set_str_binary (a, "0.110110101111011001110000111111100011101000111011101E-1073741823");
+  MPFR_ASSERTN(mpfr_cmp0 (answer, a) == 0);
+
+  mpfr_clear (a);
+  mpfr_clear (exp);
+  mpfr_clear (answer);
+}
+
+static void
+bug20080904 (void)
+{
+  mpz_t exp;
+  mpfr_t a, answer;
+  mp_exp_t emin_default;
+
+  mpz_init (exp);
+  mpfr_init2 (a, 70);
+  mpfr_init2 (answer, 70);
+
+  emin_default = mpfr_get_emin ();
+  mpfr_set_emin (MPFR_EMIN_MIN);
+
+  mpz_set_str (exp, "-4eb92f8c7b7bf81e", 16);
+  mpfr_set_str_binary (a, "1.110000101110100110100011111000011110111101000011111001111001010011100");
+
+  mpfr_pow_z (answer, a, exp, GMP_RNDN);
+  /* The correct result is near 2^(-2^62), so it underflows when
+     MPFR_EMIN_MIN > -2^62 (i.e. with 32 and 64 bits machines). */
+  mpfr_set_str (a, "AA500C0D7A69275DBp-4632850503556296886", 16, GMP_RNDN);
+  MPFR_ASSERTN(mpfr_cmp0 (answer, a) == 0);
+
+  mpfr_set_emin (emin_default);
+
+  mpz_clear (exp);
+  mpfr_clear (a);
+  mpfr_clear (answer);
+}
+
 int
 main (void)
 {
-  MPFR_TEST_USE_RANDS ();
   tests_start_mpfr ();
 
   check_special ();
+
   check_integer (2, 163, 100);
   check_regression ();
   bug20071104 ();
+  bug20080223 ();
+  bug20080904 ();
   check_overflow ();
 
   tests_end_mpfr ();
