@@ -1,30 +1,26 @@
 /* Test file for mpfr_set_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
-#include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#include <limits.h>
-#include <time.h>
 
 #include "mpfr-test.h"
 
@@ -35,7 +31,7 @@ MA 02110-1301, USA. */
   mpfr_set_str_binary (x, t); \
   if (mpfr_cmp (x, y)) \
     { \
-      printf ("Error in mpfr_set_str (%u):\n", n); \
+      printf ("Error in mpfr_set_str (%d):\n", n); \
       mpfr_print_binary (x); \
       puts (""); \
       mpfr_print_binary (y); \
@@ -82,6 +78,24 @@ check_underflow (void)
   mpfr_clear (a);
 }
 
+/* Bug found by Christoph Lauter. */
+static void
+bug20081028 (void)
+{
+  mpfr_t x;
+  const char *s = "0.10000000000000000000000000000001E1";
+
+  mpfr_init2 (x, 32);
+  mpfr_set_str (x, "1.00000000000000000006", 10, GMP_RNDU);
+  if (! mpfr_greater_p (x, __gmpfr_one))
+    {
+      printf ("Error in bug20081028:\nExpected %s\nGot      ", s);
+      mpfr_dump (x);
+      exit (1);
+    }
+  mpfr_clear (x);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -89,17 +103,18 @@ main (int argc, char *argv[])
   unsigned long k, bd, nc, i;
   char *str, *str2;
   mp_exp_t e;
-  int base, logbase, prec, baseprec, ret;
+  int base, logbase, prec, baseprec, ret, obase;
 
   tests_start_mpfr ();
 
-  if (argc >= 2) /* tset_str <string> [<prec>] [<base>] */
+  if (argc >= 2) /* tset_str <string> [<prec>] [<ibase>] [<obase>] */
     {
       prec = (argc >= 3) ? atoi (argv[2]) : 53;
       base = (argc >= 4) ? atoi (argv[3]) : 2;
+      obase = (argc >= 5) ? atoi (argv[4]) : 10;
       mpfr_init2 (x, prec);
       mpfr_set_str (x, argv[1], base, GMP_RNDN);
-      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN);
+      mpfr_out_str (stdout, obase, 0, x, GMP_RNDN);
       puts ("");
       mpfr_clear (x);
       return 0;
@@ -201,8 +216,10 @@ main (int argc, char *argv[])
   mpfr_set_prec (y, prec);
   for (i=0;i<N;i++)
     {
-      mpfr_random (x);
-      k = RND_RAND ();
+      mp_rnd_t rnd;
+
+      mpfr_urandomb (x, RANDS);
+      rnd = RND_RAND ();
       logbase = (randlimb () % 5) + 1;
       base = 1 << logbase;
       /* Warning: the number of bits needed to print exactly a number of
@@ -213,13 +230,13 @@ main (int argc, char *argv[])
         baseprec = prec;
       else
         baseprec = 1 + (prec - 2 + logbase) / logbase;
-      str = mpfr_get_str (NULL, &e, base, baseprec, x, (mp_rnd_t) k);
-      mpfr_set_str (y, str, base, (mp_rnd_t) k);
+      str = mpfr_get_str (NULL, &e, base, baseprec, x, rnd);
+      mpfr_set_str (y, str, base, rnd);
       MPFR_EXP(y) += logbase * (e - strlen (str));
       if (mpfr_cmp (x, y))
         {
           printf ("mpfr_set_str o mpfr_get_str <> id for rnd_mode=%s\n",
-                  mpfr_print_rnd_mode ((mp_rnd_t) k));
+                  mpfr_print_rnd_mode (rnd));
           printf ("x=");
           mpfr_print_binary (x);
           puts ("");
@@ -846,6 +863,7 @@ main (int argc, char *argv[])
   mpfr_clear (y);
 
   check_underflow ();
+  bug20081028 ();
 
   tests_end_mpfr ();
   return 0;
