@@ -72,21 +72,37 @@ check_sprintf (const char *expected, const char *fmt, mpfr_srcptr x)
 
   /* test mpfr_snprintf */
   p = (int) (randlimb () % n0);
-  n1 = mpfr_snprintf (buffer, p, fmt, x);
-  if ((p != 0 && n0 != n1) || (p == 0 && n1 != 0))
+  if (p == 0 && (randlimb () & 1) == 0)
+    {
+      n1 = mpfr_snprintf (NULL, 0, fmt, x);
+    }
+  else
+    {
+      buffer[p] = 17;
+      n1 = mpfr_snprintf (buffer, p, fmt, x);
+      if (buffer[p] != 17)
+        {
+          printf ("Buffer overflow in mpfr_snprintf for p = %d!\n", p);
+          exit (1);
+        }
+    }
+  if (n0 != n1)
     {
       printf ("Error in mpfr_snprintf (s, %d, \"%s\", x) return value\n",
               p, fmt);
       printf ("expected: %d\ngot:      %d\n", n0, n1);
       exit (1);
     }
-  if (strncmp (expected, buffer, p) != 0)
+  if ((p > 1 && strncmp (expected, buffer, p-1) != 0)
+      || (p == 1 && buffer[0] != '\0'))
     {
-      printf ("Error in mpfr_snprintf (s, %d, \"%s\", x);\n", p, fmt);
-      printf ("expected: \"%s\"\ngot:      \"%s\"\n", expected, buffer);
+      char part_expected[BUF_SIZE];
+      strncpy (part_expected, expected, p);
+      part_expected[p-1] = '\0';
+      printf ("Error in mpfr_vsnprintf (s, %d, \"%s\", ...);\n", p, fmt);
+      printf ("expected: \"%s\"\ngot:      \"%s\"\n", part_expected, buffer);
       exit (1);
     }
-
   return n0;
 }
 
@@ -116,8 +132,21 @@ check_vsprintf (const char *expected, const char *fmt, ...)
 
   /* test mpfr_snprintf */
   p = (int) (randlimb () % n0);
-  n1 = mpfr_vsnprintf (buffer, p, fmt, ap1);
-  if ((p != 0 && n0 != n1) || (p == 0 && n1 != 0))
+  if (p == 0 && (randlimb () & 1) == 0)
+    {
+      n1 = mpfr_vsnprintf (NULL, 0, fmt, ap1);
+    }
+  else
+    {
+      buffer[p] = 17;
+      n1 = mpfr_vsnprintf (buffer, p, fmt, ap1);
+      if (buffer[p] != 17)
+        {
+          printf ("Buffer overflow in mpfr_vsnprintf for p = %d!\n", p);
+          exit (1);
+        }
+    }
+  if (n0 != n1)
     {
       printf ("Error in mpfr_vsnprintf (s, %d, \"%s\", ...) return value\n",
               p, fmt);
@@ -126,10 +155,14 @@ check_vsprintf (const char *expected, const char *fmt, ...)
       va_end (ap1);
       exit (1);
     }
-  if (strncmp (expected, buffer, p) != 0)
+  if ((p > 1 && strncmp (expected, buffer, p-1) != 0)
+      || (p == 1 && buffer[0] != '\0'))
     {
+      char part_expected[BUF_SIZE];
+      strncpy (part_expected, expected, p);
+      part_expected[p-1] = '\0';
       printf ("Error in mpfr_vsnprintf (s, %d, \"%s\", ...);\n", p, fmt);
-      printf ("expected: \"%s\"\ngot:      \"%s\"\n", expected, buffer);
+      printf ("expected: \"%s\"\ngot:      \"%s\"\n", part_expected, buffer);
 
       va_end (ap1);
       exit (1);
@@ -174,6 +207,9 @@ native_types (void)
   sprintf (buf, "%s", s);
   check_vsprintf (buf, "%s", s);
 
+  sprintf (buf, "--%s++", "");
+  check_vsprintf (buf, "--%s++", "");
+
   sprintf (buf, "%u", ui);
   check_vsprintf (buf, "%u", ui);
 
@@ -192,6 +228,7 @@ decimal (void)
 
   /* specifier 'P' for precision */
   check_vsprintf ("128", "%Pu", p);
+  check_vsprintf ("00128", "%.5Pu", p);
 
   /* special numbers */
   mpfr_set_inf (x, 1);
@@ -228,7 +265,7 @@ decimal (void)
   /* simplest case right justified */
   check_sprintf ("      1.899347461279296875e+07", "%30Re", x);
   check_sprintf ("                         2e+07", "%30.0Re", x);
-  check_sprintf ("          18993474.61279296875", "%30Rf", x);
+  check_sprintf ("               18993474.612793", "%30Rf", x);
   check_sprintf ("              18993474.6127930", "%30.7Rf", x);
   check_sprintf ("                   1.89935e+07", "%30Rg", x);
   check_sprintf ("                         2e+07", "%30.0Rg", x);
@@ -283,32 +320,53 @@ decimal (void)
 
 
   /* neighborhood of 1 */
-  mpfr_set_str (x, "0.9999", 10, GMP_RNDN);
-  check_sprintf ("1E+00     ", "%-10.0RE", x);
-  check_sprintf ("1.0E+00   ", "%-10.1RE", x);
-  check_sprintf ("9.9990E-01", "%-10.4RE", x);
-  check_sprintf ("1.0       ", "%-10.1RF", x);
-  check_sprintf ("0.9999    ", "%-10.4RF", x);
-  check_sprintf ("1         ", "%-10.0RG", x);
-  check_sprintf ("1         ", "%-10.1RG", x);
-  check_sprintf ("0.9999    ", "%-10.4RG", x);
-  check_sprintf ("1.        ", "%-#10.0RG", x);
-  check_sprintf ("1.        ", "%-#10.1RG", x);
-  check_sprintf ("1.0       ", "%-#10.2RG", x);
-  check_sprintf ("0.9999    ", "%-#10.4RG", x);
+  mpfr_set_str (x, "0.99993896484375", 10, GMP_RNDN);
+  check_sprintf ("9.9993896484375E-01 ", "%-20RE", x);
+  check_sprintf ("9.9993896484375E-01 ", "%-20.RE", x);
+  check_sprintf ("1E+00               ", "%-20.0RE", x);
+  check_sprintf ("1.0E+00             ", "%-20.1RE", x);
+  check_sprintf ("1.00E+00            ", "%-20.2RE", x);
+  check_sprintf ("9.999E-01           ", "%-20.3RE", x);
+  check_sprintf ("9.9994E-01          ", "%-20.4RE", x);
+  check_sprintf ("0.999939            ", "%-20RF", x);
+  check_sprintf ("0.999939            ", "%-20.RF", x);
+  check_sprintf ("1                   ", "%-20.0RF", x);
+  check_sprintf ("1.0                 ", "%-20.1RF", x);
+  check_sprintf ("1.00                ", "%-20.2RF", x);
+  check_sprintf ("1.000               ", "%-20.3RF", x);
+  check_sprintf ("0.9999              ", "%-20.4RF", x);
+  check_sprintf ("0.999939            ", "%-#20RF", x);
+  check_sprintf ("0.999939            ", "%-#20.RF", x);
+  check_sprintf ("1.                  ", "%-#20.0RF", x);
+  check_sprintf ("1.0                 ", "%-#20.1RF", x);
+  check_sprintf ("1.00                ", "%-#20.2RF", x);
+  check_sprintf ("1.000               ", "%-#20.3RF", x);
+  check_sprintf ("0.9999              ", "%-#20.4RF", x);
+  check_sprintf ("1                   ", "%-20.0RG", x);
+  check_sprintf ("1                   ", "%-20.1RG", x);
+  check_sprintf ("1                   ", "%-20.2RG", x);
+  check_sprintf ("1                   ", "%-20.3RG", x);
+  check_sprintf ("0.9999              ", "%-20.4RG", x);
+  check_sprintf ("0.999939            ", "%-#20RG", x);
+  check_sprintf ("0.999939            ", "%-#20.RG", x);
+  check_sprintf ("1.                  ", "%-#20.0RG", x);
+  check_sprintf ("1.                  ", "%-#20.1RG", x);
+  check_sprintf ("1.0                 ", "%-#20.2RG", x);
+  check_sprintf ("1.00                ", "%-#20.3RG", x);
+  check_sprintf ("0.9999              ", "%-#20.4RG", x);
 
   /* multiple of 10 */
   mpfr_set_str (x, "1e17", 10, GMP_RNDN);
   check_sprintf ("1e+17", "%Re", x);
   check_sprintf ("1.000e+17", "%.3Re", x);
-  check_sprintf ("100000000000000000", "%Rf", x);
+  check_sprintf ("100000000000000000", "%.0Rf", x);
   check_sprintf ("100000000000000000.0", "%.1Rf", x);
-  check_sprintf ("100000000000000000", "%'Rf", x);
+  check_sprintf ("100000000000000000.000000", "%'Rf", x);
   check_sprintf ("100000000000000000.0", "%'.1Rf", x);
 
   mpfr_ui_div (x, 1, x, GMP_RNDN); /* x=1e-17 */
   check_sprintf ("1e-17", "%Re", x);
-  check_sprintf ("0.00000000000000001", "%Rf", x);
+  check_sprintf ("0.000000", "%Rf", x);
   check_sprintf ("1e-17", "%Rg", x);
   check_sprintf ("0.0", "%.1RDf", x);
   check_sprintf ("0.1", "%.1RUf", x);
@@ -322,11 +380,13 @@ decimal (void)
   check_sprintf ("0.008", "%.3RF", x);
   check_sprintf ("0.008", "%.3RUF", x);
 
-  /* limit test for the choice beetwen %f-style and %g-style */
+  /* check limit between %f-style and %g-style */
   mpfr_set_str (x, "0.0000999", 10, GMP_RNDN);
-  check_sprintf ("0.0001", "%.0Rg", x);
-  check_sprintf ("9e-05", "%.0RDg", x);
-  check_sprintf ("0.0001", "%.2Rg", x);
+  check_sprintf ("0.0001",   "%.0Rg", x);
+  check_sprintf ("9e-05",    "%.0RDg", x);
+  check_sprintf ("0.0001",   "%.1Rg", x);
+  check_sprintf ("0.0001",   "%.2Rg", x);
+  check_sprintf ("9.99e-05", "%.3Rg", x);
 
   /* trailing zeros */
   mpfr_set_si_2exp (x, -1, -15, GMP_RNDN); /* x=-2^-15 */
@@ -358,6 +418,68 @@ decimal (void)
   mpfr_set_str (x, "-9.90597761233942053494e-01", 10, GMP_RNDN);
   check_sprintf ("-1.", "%- #0.1RG", x);
 
+  /* precision zero */
+  mpfr_set_d (x, -9.5, GMP_RNDN);
+  check_sprintf ("-10",    "%.0RDf", x);
+  check_sprintf ("-10",    "%.0Rf", x);
+  check_sprintf ("-1e+01", "%.0Re", x);
+  check_sprintf ("-1e+01", "%.0Rg", x);
+  mpfr_set_ui_2exp (x, 1, -1, GMP_RNDN);
+  check_sprintf ("0",      "%.0Rf", x);
+  check_sprintf ("5e-01",  "%.0Re", x);
+  check_sprintf ("0.5",    "%.0Rg", x);
+  mpfr_set_ui_2exp (x, 3, -1, GMP_RNDN);
+  check_sprintf ("2",      "%.0Rf", x);
+  mpfr_set_ui_2exp (x, 5, -1, GMP_RNDN);
+  check_sprintf ("2",      "%.0Rf", x);
+  mpfr_set_ui (x, 0x1f, GMP_RNDN);
+  check_sprintf ("0x1p+5", "%.0Ra", x);
+  mpfr_set_ui (x, 3, GMP_RNDN);
+  check_sprintf ("1p+2",   "%.0Rb", x);
+
+  /* round to next ten power with %f but not with %g */
+  mpfr_set_str (x, "-6.64464380544039223686e-02", 10, GMP_RNDN);
+  check_sprintf ("-0.1",  "%.1Rf", x);
+  check_sprintf ("-0.0",  "%.1RZf", x);
+  check_sprintf ("-0.07", "%.1Rg", x);
+  check_sprintf ("-0.06", "%.1RZg", x);
+
+  /* round to next ten power and do not remove trailing zeros */
+  mpfr_set_str (x, "9.98429393291486722006e-02", 10, GMP_RNDN);
+  check_sprintf ("0.1",   "%#.1Rg", x);
+  check_sprintf ("0.10",  "%#.2Rg", x);
+  check_sprintf ("0.099", "%#.2RZg", x);
+
+  /* Halfway cases */
+  mpfr_set_str (x, "1.5", 10, GMP_RNDN);
+  check_sprintf ("2e+00", "%.0Re", x);
+  mpfr_set_str (x, "2.5", 10, GMP_RNDN);
+  check_sprintf ("2e+00", "%.0Re", x);
+  mpfr_set_str (x, "9.5", 10, GMP_RNDN);
+  check_sprintf ("1e+01", "%.0Re", x);
+  mpfr_set_str (x, "1.25", 10, GMP_RNDN);
+  check_sprintf ("1.2e+00", "%.1Re", x);
+  mpfr_set_str (x, "1.75", 10, GMP_RNDN);
+  check_sprintf ("1.8e+00", "%.1Re", x);
+  mpfr_set_str (x, "-0.5", 10, GMP_RNDN);
+  check_sprintf ("-0", "%.0Rf", x);
+  mpfr_set_str (x, "1.25", 10, GMP_RNDN);
+  check_sprintf ("1.2", "%.1Rf", x);
+  mpfr_set_str (x, "1.75", 10, GMP_RNDN);
+  check_sprintf ("1.8", "%.1Rf", x);
+  mpfr_set_str (x, "1.5", 10, GMP_RNDN);
+  check_sprintf ("2", "%.1Rg", x);
+  mpfr_set_str (x, "2.5", 10, GMP_RNDN);
+  check_sprintf ("2", "%.1Rg", x);
+  mpfr_set_str (x, "9.25", 10, GMP_RNDN);
+  check_sprintf ("9.2", "%.2Rg", x);
+  mpfr_set_str (x, "9.75", 10, GMP_RNDN);
+  check_sprintf ("9.8", "%.2Rg", x);
+
+  /* assertion failure in r6320 */
+  mpfr_set_str (x, "-9.996", 10, GMP_RNDN);
+  check_sprintf ("-10.0", "%.1Rf", x);
+
   mpfr_clears (x, z, (mpfr_ptr) 0);
   return 0;
 }
@@ -387,23 +509,23 @@ hexadecimal (void)
 
   /* simplest case right justified */
   check_sprintf ("   0xf.edcba987654321p+24", "%25Ra", x);
-  check_sprintf ("                  0x8p+25", "%25.0Ra", x);
+  check_sprintf ("                  0x1p+28", "%25.0Ra", x);
   check_sprintf ("                   0x0p+0", "%25.0Ra", z);
   /* sign or space, pad with leading zeros */
   check_sprintf (" 0X00F.EDCBA987654321P+24", "% 025RA", x);
-  check_sprintf (" 0X000000000000000008P+25", "% 025.0RA", x);
+  check_sprintf (" 0X000000000000000001P+28", "% 025.0RA", x);
   check_sprintf (" 0X0000000000000000000P+0", "% 025.0RA", z);
   /* sign + or -, left justified */
   check_sprintf ("+0xf.edcba987654321p+24  ", "%+-25Ra", x);
-  check_sprintf ("+0x8p+25                 ", "%+-25.0Ra", x);
+  check_sprintf ("+0x1p+28                 ", "%+-25.0Ra", x);
   check_sprintf ("+0x0p+0                  ", "%+-25.0Ra", z);
   /* decimal point, left justified, precision and rounding parameter */
   check_vsprintf ("0XF.FP+24 ", "%#-10.*R*A", 1, GMP_RNDN, x);
-  check_vsprintf ("0X8.P+25  ", "%#-10.*R*A", 0, GMP_RNDN, x);
+  check_vsprintf ("0X1.P+28  ", "%#-10.*R*A", 0, GMP_RNDN, x);
   check_vsprintf ("0X0.P+0   ", "%#-10.*R*A", 0, GMP_RNDN, z);
   /* sign or space */
   check_sprintf (" 0xf.eddp+24", "% .3RNa", x);
-  check_sprintf (" 0x8p+25",     "% .0RNa", x);
+  check_sprintf (" 0x1p+28",     "% .0RNa", x);
   /* sign + or -, decimal point, pad with leading zeros */
   check_sprintf ("+0X0F.EP+24", "%0+#11.1RZA", x);
   check_sprintf ("+0X00F.P+24", "%0+#11.0RZA", x);
@@ -428,11 +550,57 @@ hexadecimal (void)
   check_sprintf ("0XFP+0", "%.0RNA", x);
   /* tie case in round to nearest mode */
   mpfr_set_str (x, "0x0.8800000000000000p+3", 16, GMP_RNDN);
-  check_sprintf ("0x8.p-1", "%#.0RNa", x);
+  check_sprintf ("0x9.p-1", "%#.0RNa", x);
   mpfr_set_str (x, "-0x0.9800000000000000p+3", 16, GMP_RNDN);
   check_sprintf ("-0xap-1", "%.0RNa", x);
   /* trailing zeros in fractional part */
   check_sprintf ("-0X4.C0000000000000000000P+0", "%.20RNA", x);
+  /* rounding bit is one and the first non zero bit is far away */
+  mpfr_set_prec (x, 1024);
+  mpfr_set_ui_2exp (x, 29, -1, GMP_RNDN);
+  mpfr_nextabove (x);
+  check_sprintf ("0XFP+0", "%.0RNA", x);
+
+  /* with more than one limb */
+  mpfr_set_prec (x, 300);
+  mpfr_set_str (x, "0xf.ffffffffffffffffffffffffffffffffffffffffffffffffffff"
+                "fffffffffffffffff", 16, GMP_RNDN);
+  check_sprintf ("0x1p+4 [300]", "%.0RNa [300]", x);
+  check_sprintf ("0xfp+0 [300]", "%.0RZa [300]", x);
+  check_sprintf ("0xfp+0 [300]", "%.0RDa [300]", x);
+  check_sprintf ("0x1p+4 [300]", "%.0RUa [300]", x);
+  check_sprintf ("0x1.0000000000000000000000000000000000000000p+4",
+                 "%.40RNa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffffffp+0",
+                 "%.40RZa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffffffp+0",
+                 "%.40RDa", x);
+  check_sprintf ("0x1.0000000000000000000000000000000000000000p+4",
+                 "%.40RUa", x);
+
+  mpfr_set_str (x, "0xf.7fffffffffffffffffffffffffffffffffffffffffffffffffff"
+                "ffffffffffffffffff", 16, GMP_RNDN);
+  check_sprintf ("0XFP+0", "%.0RNA", x);
+  check_sprintf ("0XFP+0", "%.0RZA", x);
+  check_sprintf ("0XFP+0", "%.0RDA", x);
+  check_sprintf ("0X1P+4", "%.0RUA", x);
+  check_sprintf ("0XF.8P+0", "%.1RNA", x);
+  check_sprintf ("0XF.7P+0", "%.1RZA", x);
+  check_sprintf ("0XF.7P+0", "%.1RDA", x);
+  check_sprintf ("0XF.8P+0", "%.1RUA", x);
+
+  /* do not round up to the next power of the base */
+  mpfr_set_str (x, "0xf.fffffffffffffffffffffffffffffffffffffeffffffffffffff"
+                "ffffffffffffffffff", 16, GMP_RNDN);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffff00p+0",
+                 "%.40RNa", x);
+  check_sprintf ("0xf.fffffffffffffffffffffffffffffffffffffeffp+0",
+                 "%.40RZa", x);
+  check_sprintf ("0xf.fffffffffffffffffffffffffffffffffffffeffp+0",
+                 "%.40RDa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffff00p+0",
+                 "%.40RUa", x);
+
 
   mpfr_clears (x, z, (mpfr_ptr) 0);
   return 0;
@@ -479,8 +647,10 @@ binary (void)
   /* pad with leading zero */
   check_sprintf ("00001.1100101011001101p+9", "%025RDb", x);
   /* sign or space, decimal point (unused), left justified */
-  check_sprintf (" 1.1p+9    " , "%- #11.1RDb", x);
-  check_sprintf (" 1.1p+9    " , "%- #11.0RDb", x);
+  check_sprintf (" 1.1p+9    ", "%- #11.1RDb", x);
+  check_sprintf (" 1.p+9     ", "%- #11.0RDb", x);
+  check_sprintf (" 1.p+10    ", "%- #11.0RNb", x);
+  check_sprintf (" 1.p+10    ", "%- #11.0RUb", x);
 
   mpfr_mul_si (x, x, -1, GMP_RNDD);
   mpfr_mul_si (z, z, -1, GMP_RNDD);
@@ -489,6 +659,17 @@ binary (void)
   check_sprintf ("   -1.1p+9", "%+10.1RUb", x);
   check_sprintf ("   -0.0p+0", "%+10.1RUb", z);
 
+  /* precision 0 */
+  check_sprintf ("-1p+10", "%.0RNb", x);
+  check_sprintf ("-1p+9",  "%.0RZb", x);
+  check_sprintf ("-1p+10", "%.0RDb", x);
+  check_sprintf ("-1p+9",  "%.0RUb", x);
+  /* round to next base power */
+  check_sprintf ("-1.0p+10", "%.1RNb", x);
+  check_sprintf ("-1.0p+10", "%.1RDb", x);
+  /* do not round to next base power */
+  check_sprintf ("-1.1p+9", "%.1RUb", x);
+  check_sprintf ("-1.1p+9", "%.1RZb", x);
   /* rounding bit is zero */
   check_sprintf ("-1.11p+9", "%.2RNb", x);
   /* tie case in round to nearest mode */
@@ -553,15 +734,18 @@ mixed (void)
   return 0;
 }
 
-/* Check with locale "da_DK"
-   decimal point is ',' and thousands separator is '.' */
+/* Check with locale "da_DK". On most platforms, decimal point is ','
+   and thousands separator is '.'; the test is not performed if this
+   is not the case or if the locale doesn't exist. */
 static int
 locale_da_DK (void)
 {
   mpfr_prec_t p = 128;
   mpfr_t x;
 
-  if (setlocale (LC_ALL, "da_DK") == 0)
+  if (setlocale (LC_ALL, "da_DK") == 0 ||
+      localeconv()->decimal_point[0] != ',' ||
+      localeconv()->thousands_sep[0] != '.')
     return 0;
 
   mpfr_init2 (x, p);
@@ -573,21 +757,21 @@ locale_da_DK (void)
   check_sprintf ("      1,899347461279296875e+07", "%'30Re", x);
   check_sprintf ("                   1,89935e+07", "%'30Rg", x);
   check_sprintf ("        18.993.474,61279296875", "%'30.19Rg", x);
-  check_sprintf ("        18.993.474,61279296875", "%'30Rf", x);
+  check_sprintf ("             18.993.474,612793", "%'30Rf", x);
 
   /* sign or space, pad, thousands separator with leading zeros */
   check_sprintf (" 000001,899347461279296875E+07", "%' 030RE", x);
   check_sprintf (" 0000000000000000001,89935E+07", "%' 030RG", x);
   check_sprintf (" 000000018.993.474,61279296875", "%' 030.19RG", x);
-  check_sprintf (" 000000018.993.474,61279296875", "%' 030RF", x);
+  check_sprintf (" 00000000000018.993.474,612793", "%' 030RF", x);
 
   mpfr_set_ui (x, 50, GMP_RNDN);
   mpfr_exp10 (x, x, GMP_RNDN);
-  check_sprintf ("100000000000000000000000000000000000000000000000000", "%Rf",
+  check_sprintf ("100000000000000000000000000000000000000000000000000", "%.0Rf",
                  x);
   check_sprintf
     ("100.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000,",
-     "%'#Rf", x);
+     "%'#.0Rf", x);
   check_sprintf
     ("100.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000,0000",
      "%'.4Rf", x);
@@ -810,6 +994,69 @@ bug20081214 (void)
   mpfr_clear (x);
 }
 
+/* In particular, the following test makes sure that the rounding
+ * for %Ra and %Rb is not done on the MPFR number itself (as it
+ * would overflow). Note: it has been reported on comp.std.c that
+ * some C libraries behave differently on %a, but this is a bug.
+ */
+static void
+check_emax_aux (mp_exp_t e)
+{
+  mpfr_t x;
+  char *s1, s2[256];
+  int i;
+  mp_exp_t emax;
+
+  MPFR_ASSERTN (e <= LONG_MAX);
+  emax = mpfr_get_emax ();
+  set_emax (e);
+
+  mpfr_init2 (x, 16);
+
+  mpfr_set_inf (x, 1);
+  mpfr_nextbelow (x);
+
+  i = mpfr_asprintf (&s1, "%Ra %.2Ra", x, x);
+  MPFR_ASSERTN (i > 0);
+
+  mpfr_snprintf (s2, 256, "0x7.fff8p+%ld 0x8.00p+%ld", e-3, e-3);
+
+  if (strcmp (s1, s2) != 0)
+    {
+      printf ("Error in check_emax_aux for emax = %ld\n", e);
+      printf ("Expected %s\n", s2);
+      printf ("Got      %s\n", s1);
+      exit (1);
+    }
+
+  mpfr_free_str (s1);
+
+  i = mpfr_asprintf (&s1, "%Rb %.2Rb", x, x);
+  MPFR_ASSERTN (i > 0);
+
+  mpfr_snprintf (s2, 256, "1.111111111111111p+%ld 1.00p+%ld", e-1, e);
+
+  if (strcmp (s1, s2) != 0)
+    {
+      printf ("Error in check_emax_aux for emax = %ld\n", e);
+      printf ("Expected %s\n", s2);
+      printf ("Got      %s\n", s1);
+      exit (1);
+    }
+
+  mpfr_free_str (s1);
+
+  mpfr_clear (x);
+  set_emax (emax);
+}
+
+static void
+check_emax (void)
+{
+  check_emax_aux (15);
+  check_emax_aux (MPFR_EMAX_MAX);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -827,6 +1074,7 @@ main (int argc, char **argv)
   binary ();
   decimal ();
   mixed ();
+  check_emax ();
 
 #if defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE)
   locale_da_DK ();
