@@ -1,55 +1,30 @@
 /* Test file for mpfr_set_str.
 
-Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
-#include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#include <limits.h>
-#include <time.h>
 
 #include "mpfr-test.h"
 
-static void check_reftable (void);
-static void check_special  (void);
-static void check_retval   (void);
-static void check_overflow (void);
-static void check_parse    (void);
-
-int
-main (int argc, char *argv[])
-{
-  tests_start_mpfr ();
-
-  check_special();
-  check_reftable ();
-  check_parse ();
-  check_overflow ();
-  check_retval ();
-
-  tests_end_mpfr ();
-  return 0;
-}
-
-void
+static void
 check_special (void)
 {
   mpfr_t x, y;
@@ -129,6 +104,13 @@ check_special (void)
       printf ("Error for setting NAN (6)\n");
       exit (1);
     }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
+  res = mpfr_strtofr (x, "NAN()foo", &s, 10, GMP_RNDN);
+  if (res != 0 || !mpfr_nan_p (x) || strcmp(s, "foo") )
+    {
+      printf ("Error for setting NAN (7)\n");
+      exit (1);
+    }
 
   /* Check INF */
   res = mpfr_strtofr (x, "INFINITY", &s, 8, GMP_RNDN);
@@ -211,7 +193,8 @@ int randomab (int a, int b)
   return a + rand () % (b-a);
 }
 
-int main()
+int
+main (void)
 {
   int i, base;
   mpfr_t x;
@@ -231,7 +214,7 @@ int main()
       base = randomab (2, 30);
       e = randomab (-1<<15, 1<<15);
       mpfr_set_prec (x, p);
-      mpfr_random (x);
+      mpfr_urandomb (x, RANDS);
       mpfr_mul_2si (x, x, e, GMP_RNDN);
       printf("{%lu, %d,\n\"", p, base);
       mpfr_out_str (stdout, base, p, x, GMP_RNDN);
@@ -552,9 +535,8 @@ static struct dymmy_test {
 "1.001000010110011011000101100000101111101001100011101101001111110111000010010110010001100e-16920"}
 };
 
-
-void
-check_reftable ()
+static void
+check_reftable (void)
 {
   int i, base;
   mpfr_t x, y;
@@ -598,7 +580,7 @@ check_reftable ()
   mpfr_clear (x);
 }
 
-void
+static void
 check_parse (void)
 {
   mpfr_t x;
@@ -952,3 +934,104 @@ check_retval (void)
 
   mpfr_clear (x);
 }
+
+/* Bug found by Christoph Lauter (in mpfr_set_str). */
+static struct bug20081025_test {
+  mpfr_rnd_t rnd;
+  int inexact;
+  const char *str;
+  const char *binstr;
+} Bug20081028Table[] = {
+  {GMP_RNDN, -1, "1.00000000000000000006", "1"},
+  {GMP_RNDZ, -1, "1.00000000000000000006", "1"},
+  {GMP_RNDU, +1, "1.00000000000000000006",
+   "10000000000000000000000000000001e-31"},
+  {GMP_RNDD, -1, "1.00000000000000000006", "1"},
+
+
+  {GMP_RNDN, +1, "-1.00000000000000000006", "-1"},
+  {GMP_RNDZ, +1, "-1.00000000000000000006", "-1"},
+  {GMP_RNDU, +1, "-1.00000000000000000006", "-1"},
+  {GMP_RNDD, -1, "-1.00000000000000000006",
+   "-10000000000000000000000000000001e-31"},
+
+  {GMP_RNDN, +1, "0.999999999999999999999999999999999999999999999", "1"},
+  {GMP_RNDZ, -1, "0.999999999999999999999999999999999999999999999",
+   "11111111111111111111111111111111e-32"},
+  {GMP_RNDU, +1, "0.999999999999999999999999999999999999999999999", "1"},
+  {GMP_RNDD, -1, "0.999999999999999999999999999999999999999999999",
+   "11111111111111111111111111111111e-32"},
+
+  {GMP_RNDN, -1, "-0.999999999999999999999999999999999999999999999", "-1"},
+  {GMP_RNDZ, +1, "-0.999999999999999999999999999999999999999999999",
+   "-11111111111111111111111111111111e-32"},
+  {GMP_RNDU, +1, "-0.999999999999999999999999999999999999999999999",
+   "-11111111111111111111111111111111e-32"},
+  {GMP_RNDD, -1, "-0.999999999999999999999999999999999999999999999", "-1"}
+};
+
+static void
+bug20081028 (void)
+{
+  int i;
+  int inexact, res;
+  mpfr_rnd_t rnd;
+  mpfr_t x, y;
+  char *s;
+
+  mpfr_init2 (x, 32);
+  mpfr_init2 (y, 32);
+  for (i = 0 ; i < numberof (Bug20081028Table) ; i++)
+    {
+      rnd     = Bug20081028Table[i].rnd;
+      inexact = Bug20081028Table[i].inexact;
+      mpfr_set_str_binary (x, Bug20081028Table[i].binstr);
+      res = mpfr_strtofr (y, Bug20081028Table[i].str, &s, 10, rnd);
+      if (s == NULL || *s != 0)
+        {
+          printf ("Error in Bug20081028: strtofr didn't parse entire input\n"
+                  "for (i=%d) Str=\"%s\"", i, Bug20081028Table[i].str);
+          exit (1);
+        }
+      if (! SAME_SIGN (res, inexact))
+        {
+          printf ("Error in Bug20081028: expected %s ternary value, "
+                  "got %d\nfor (i=%d) Rnd=%s Str=\"%s\"\n Set binary gives: ",
+                  inexact > 0 ? "positive" : "negative",
+                  res, i, mpfr_print_rnd_mode(rnd), Bug20081028Table[i].str);
+          mpfr_dump (x);
+          printf (" strtofr    gives: ");
+          mpfr_dump (y);
+          exit (1);
+        }
+      if (mpfr_cmp (x, y))
+        {
+          printf ("Error in Bug20081028: Results differ between strtofr and "
+                  "set_binary\nfor (i=%d) Rnd=%s Str=\"%s\"\n"
+                  " Set binary gives: ",
+                  i, mpfr_print_rnd_mode(rnd), Bug20081028Table[i].str);
+          mpfr_dump (x);
+          printf (" strtofr    gives: ");
+          mpfr_dump (y);
+          exit (1);
+        }
+    }
+  mpfr_clear (y);
+  mpfr_clear (x);
+}
+
+int
+main (int argc, char *argv[])
+{
+  tests_start_mpfr ();
+
+  check_special();
+  check_reftable ();
+  check_parse ();
+  check_overflow ();
+  check_retval ();
+  bug20081028 ();
+
+  tests_end_mpfr ();
+  return 0;
+}
