@@ -1,29 +1,28 @@
 /* Test file for mpfr_ui_sub.
 
-Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <float.h>
-#include <time.h>
 
 #include "mpfr-test.h"
 
@@ -123,7 +122,7 @@ special (void)
 
   mpfr_set_prec (x, 10);
   mpfr_set_prec (y, 10);
-  mpfr_random (x);
+  mpfr_urandomb (x, RANDS);
   mpfr_ui_sub (y, 0, x, GMP_RNDN);
   if (MPFR_IS_ZERO(x))
     MPFR_ASSERTN(MPFR_IS_ZERO(y));
@@ -147,7 +146,7 @@ check (unsigned long y, const char *xs, mp_rnd_t rnd_mode, const char *zs)
 {
   mpfr_t xx, zz;
 
-  mpfr_inits2 (53, xx, zz, (void *) 0);
+  mpfr_inits2 (53, xx, zz, (mpfr_ptr) 0);
   mpfr_set_str1 (xx, xs);
   mpfr_ui_sub (zz, y, xx, rnd_mode);
   if (mpfr_cmp_str1 (zz, zs) )
@@ -158,7 +157,7 @@ check (unsigned long y, const char *xs, mp_rnd_t rnd_mode, const char *zs)
               y, xs, mpfr_print_rnd_mode (rnd_mode));
       exit (1);
     }
-  mpfr_clears (xx, zz, (void *) 0);
+  mpfr_clears (xx, zz, (mpfr_ptr) 0);
 }
 
 /* if u = o(x-y), v = o(u-x), w = o(v+y), then x-y = u-w */
@@ -170,13 +169,13 @@ check_two_sum (mp_prec_t p)
   mp_rnd_t rnd;
   int inexact;
 
-  mpfr_inits2 (p, y, u, v, w, (void *) 0);
+  mpfr_inits2 (p, y, u, v, w, (mpfr_ptr) 0);
   do
     {
       x = randlimb ();
     }
   while (x < 1);
-  mpfr_random (y);
+  mpfr_urandomb (y, RANDS);
   rnd = GMP_RNDN;
   inexact = mpfr_ui_sub (u, x, y, rnd);
   mpfr_sub_ui (v, u, x, rnd);
@@ -196,7 +195,7 @@ check_two_sum (mp_prec_t p)
       printf ("inexact = %d\n", inexact);
       exit (1);
     }
-  mpfr_clears (y, u, v, w, (void *) 0);
+  mpfr_clears (y, u, v, w, (mpfr_ptr) 0);
 }
 
 static void
